import 'package:flutter/material.dart';
import 'package:fl_chart/fl_chart.dart';

import '../../../config/app_colors.dart';
import '../../../models/topartists.dart';
import '../../../utils/color_extensions.dart';

class ChartTopArtists extends StatelessWidget {
  final TopArtistsData chartData;

  ChartTopArtists({super.key, required this.chartData});

  @override
  Widget build(BuildContext context) {
    return AspectRatio(
      aspectRatio: 2.1,
      child: Row(
        children: <Widget>[
          Expanded(
            child: AspectRatio(
              aspectRatio: 1,
              child: PieChart(
                PieChartData(
                  sections: getPieChartData(),
                  sectionsSpace: 1,
                  centerSpaceRadius: 40,
                  startDegreeOffset: -45,
                  pieTouchData: PieTouchData(enabled: false),
                  borderData: FlBorderData(show: false),
                ),
              ),
            ),
          ),
          buildLegendWidget(),
        ],
      ),
    );
  }

  Color getColorIndex(int index) {
    const List<int> hexValues = [
      0x8dd3c7,
      0xffffb3,
      0xbebada,
      0xfb8072,
      0x80b1d3,
      0xfdb462,
      0xb3de69,
      0xfccde5,
      0xd9d9d9,
      0xbc80bd,
      0xccebc5,
      0xffed6f,
    ];

    return Color(hexValues[index] + 0xff000000);
  }

  List<PieChartSectionData> getPieChartData() {
    List<PieChartSectionData> items = [];

    final radius = 40.0;
    final fontSize = 11.0;
    const shadows = [Shadow(color: Colors.black, blurRadius: 2)];

    int index = 0;

    this.chartData.topArtists.forEach((element) {
      items.add(PieChartSectionData(
        value: element.count.toDouble(),
        title: element.artistName,
        color: getColorIndex(index++).darken(20),
        radius: radius,
        titleStyle: TextStyle(
          fontSize: fontSize,
          color: AppColors.mainTextColor1,
          shadows: shadows,
        ),
      ));
    });

    return items;
  }

  Widget buildLegendWidget() {
    const double itemSize = 12;

    List<Widget> items = [];
    int index = 0;

    this.chartData.topArtists.forEach((element) {
      items.add(Row(
        children: <Widget>[
          Container(
            width: itemSize,
            height: itemSize,
            decoration: BoxDecoration(
              shape: BoxShape.rectangle,
              color: getColorIndex(index++).darken(20),
            ),
          ),
          const SizedBox(
            width: 4,
          ),
          Text(
            element.artistName + ' (' + element.count.toString() + ')',
            style: TextStyle(
              fontSize: itemSize - 2,
              fontWeight: FontWeight.bold,
            ),
          )
        ],
      ));
    });

    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: items,
    );
  }
}
